package com.ningdatech.pmapi.todocenter.manage;

import static com.ningdatech.pmapi.todocenter.constant.WorkNotice.*;
import static com.wflow.workflow.task.TriggerServiceTask.runtimeService;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.FileUtils;
import org.assertj.core.util.Lists;
import org.flowable.bpmn.model.*;
import org.flowable.engine.HistoryService;
import org.flowable.engine.history.HistoricProcessInstance;
import org.flowable.engine.runtime.ActivityInstance;
import org.springframework.beans.BeanUtils;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.basic.util.NdDateUtils;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.file.entity.File;
import com.ningdatech.file.entity.vo.result.FileResultVO;
import com.ningdatech.file.service.FileService;
import com.ningdatech.pmapi.common.constant.CommonConst;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.common.model.entity.ExcelExportWriter;
import com.ningdatech.pmapi.common.statemachine.util.StateMachineUtils;
import com.ningdatech.pmapi.common.util.ExcelDownUtil;
import com.ningdatech.pmapi.irs.sign.IRSAPIRequest;
import com.ningdatech.pmapi.organization.model.entity.DingEmployeeInfo;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingEmployeeInfoService;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import com.ningdatech.pmapi.projectdeclared.manage.DefaultDeclaredProjectManage;
import com.ningdatech.pmapi.projectlib.enumeration.FourSystemEnum;
import com.ningdatech.pmapi.projectlib.enumeration.InstTypeEnum;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectTypeEnum;
import com.ningdatech.pmapi.projectlib.manage.ProjectLibManage;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectApplication;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectInst;
import com.ningdatech.pmapi.projectlib.model.req.ProjectListReq;
import com.ningdatech.pmapi.projectlib.service.IProjectApplicationService;
import com.ningdatech.pmapi.projectlib.service.IProjectInstService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.signature.entity.CompanySignature;
import com.ningdatech.pmapi.signature.service.ICompanySignatureService;
import com.ningdatech.pmapi.staging.enums.MsgTypeEnum;
import com.ningdatech.pmapi.staging.service.INdWorkNoticeStagingService;
import com.ningdatech.pmapi.staging.service.IProjectStagingService;
import com.ningdatech.pmapi.sys.model.entity.Notify;
import com.ningdatech.pmapi.sys.service.INotifyService;
import com.ningdatech.pmapi.todocenter.bean.entity.WorkNoticeInfo;
import com.ningdatech.pmapi.todocenter.enumeration.IsAppendProjectEnum;
import com.ningdatech.pmapi.todocenter.enumeration.IsOrNotEnum;
import com.ningdatech.pmapi.todocenter.model.dto.AdjustHandleDTO;
import com.ningdatech.pmapi.todocenter.model.dto.PdfGenerateDTO;
import com.ningdatech.pmapi.todocenter.model.dto.SealInfoDTO;
import com.ningdatech.pmapi.todocenter.model.dto.SignReqDTO;
import com.ningdatech.pmapi.todocenter.model.po.TodoCenterStatisticsPO;
import com.ningdatech.pmapi.todocenter.model.req.ProcessDetailReq;
import com.ningdatech.pmapi.todocenter.model.req.ToBeProcessedExportReq;
import com.ningdatech.pmapi.todocenter.model.req.ToBeProcessedReq;
import com.ningdatech.pmapi.todocenter.model.vo.ProcessProgressDetailVo;
import com.ningdatech.pmapi.todocenter.model.vo.ResToBeProcessedVO;
import com.ningdatech.pmapi.todocenter.model.vo.TodoCenterStatisticsVO;
import com.ningdatech.pmapi.todocenter.service.StatisticsService;
import com.ningdatech.pmapi.todocenter.utils.BuildUserUtils;
import com.ningdatech.pmapi.todocenter.utils.PdfUtils;
import com.ningdatech.pmapi.user.entity.UserInfo;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.service.IUserInfoService;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import com.wflow.contants.HisProInsEndActId;
import com.wflow.contants.ProcessConstant;
import com.wflow.exception.BusinessException;
import com.wflow.workflow.bean.dto.ProcessInstanceUserDto;
import com.wflow.workflow.bean.dto.ReqProcessHandlerDTO;
import com.wflow.workflow.bean.dto.TodoCenterListReqDTO;
import com.wflow.workflow.bean.process.ProgressNode;
import com.wflow.workflow.bean.process.enums.NodeTypeEnum;
import com.wflow.workflow.bean.vo.ProcessInstanceVo;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.bean.vo.ProcessTaskVo;
import com.wflow.workflow.enums.ProcessHandlerEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import com.wflow.workflow.service.ProcessTaskService;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.codec.Base64;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * @author ZPF
 * @since 2023/02/19 16:09
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class TodoCenterManage {

    private final ProcessTaskService processTaskService;
    private final HistoryService historyService;
    private final IUserInfoService userInfoService;
    private final IProjectService projectService;
    private final ProjectLibManage projectLibManage;
    private final StateMachineUtils stateMachineUtils;
    private final IDingEmployeeInfoService dingEmployeeInfoService;
    private final IDingOrganizationService dingOrganizationService;
    private final ProcessInstanceService processInstanceService;
    private final INdWorkNoticeStagingService workNoticeStagingService;
    private final DefaultDeclaredProjectManage defaultDeclaredProjectManage;
    private final IProjectApplicationService projectApplicationService;
    private final UserInfoHelper userInfoHelper;
    private final BuildUserUtils buildUserUtils;
    private final IProjectStagingService projectStagingService;
    private final IProjectInstService projectInstService;
    private final PdfUtils pdfUtils;
    private final FileService fileService;
    private final ICompanySignatureService companySignatureService;
    private final StatisticsService statisticsService;
    private final INotifyService notifyService;

    /**
     * 待办中心待我处理项目列表查询
     *
     * @param param
     * @author zpf
     * @since 2023/02/18
     */
    public PageVo<ResToBeProcessedVO> todoProjectList(ToBeProcessedReq param) {
        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取员工浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
        // 再查出项目关联的流程实例ID
        List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
        if(CollUtil.isEmpty(projectIdList)){
            return PageVo.empty();
        }
        List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIdList)
                .orderByDesc(ProjectInst::getProjectId));
        Map<String, Project> projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
        List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());
        if(CollUtil.isEmpty(instCodes)){
            return PageVo.empty();
        }
        // 查出用户工作流
        TodoCenterListReqDTO req = new TodoCenterListReqDTO();
        req.setInstCodes(instCodes);
        Integer pageNumber = param.getPageNumber();
        Integer pageSize = param.getPageSize();
        req.setProcessDefId(param.getProcessDefId());
        req.setUserId(String.valueOf(userId));
        req.setEmployeeCode(employeeCode);
        List<ProcessTaskVo> todoList = processTaskService.getUserTodoList(req);

        if (CollUtil.isEmpty(todoList)) {
            return PageVo.empty();
        }

        List<ProcessTaskVo> userTodoList = todoList.stream()
                .skip((long) (pageNumber - 1) * pageSize)
                .limit(pageSize)
                .collect(Collectors.toList());

        Set<String> processSet = Sets.newHashSet();
        Map<String, ProcessTaskVo> taskMap = userTodoList.stream()
                .sorted((t1,t2) -> t2.getTaskCreateTime().compareTo(t1.getTaskCreateTime()))
                .collect(Collectors.toList()).stream()
                .filter(v -> processSet.add(v.getInstanceId()))
                .collect(Collectors.toMap(ProcessTaskVo::getInstanceId, v -> v));

        List<ResToBeProcessedVO> resVos = userTodoList.stream().map(d -> {
            Project projectInfo = projectInfoMap.get(d.getInstanceId());
            ResToBeProcessedVO res = new ResToBeProcessedVO();
            BeanUtils.copyProperties(projectInfo, res);
            res.setProjectId(projectInfo.getId());
            res.setBuildOrg(projectInfo.getBuildOrgName());
            res.setDeclaredAmount(projectInfo.getDeclareAmount());
            ProcessTaskVo taskVo = taskMap.get(d.getInstanceId());
            res.setNodeId(taskVo.getTaskDefKey());
            res.setProcessStatusName(taskVo.getStatus());
            res.setProcessLaunchTime(d.getCreateTime());
            return res;
        }).collect(Collectors.toList());
        return PageVo.of(resVos, todoList.size());
    }

    /**
     * 待办中心列表导出
     *
     * @param response
     * @param param
     * @return void
     * @author CMM
     * @since 2023/02/01
     */
    public void exportTodoList(HttpServletResponse response, ToBeProcessedExportReq param) {
        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取用户全量用户信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
        // 再查出项目关联的流程实例ID
        List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
        List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIdList)
                .orderByDesc(ProjectInst::getProjectId));
        Map<String, Project> projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
        List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

        List<ProcessTaskVo> userTodoList = Lists.newArrayList();
        if (CollUtil.isNotEmpty(instCodes)) {
            // 查出用户工作流
            TodoCenterListReqDTO req = new TodoCenterListReqDTO();
            req.setInstCodes(instCodes);
            req.setProcessDefId(param.getProcessDefId());
            req.setUserId(String.valueOf(userId));
            req.setEmployeeCode(employeeCode);
            BeanUtils.copyProperties(param,req);
            userTodoList = processTaskService.getUserTodoList(req);
        }

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();
        String fileName = null;
        if (IsAppendProjectEnum.APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_待我处理_增补项目列表";
        } else if (IsAppendProjectEnum.NOT_APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_待我处理_非增补项目列表";
        }
        excelExportWriter.setFileName(fileName);
        List<String> sheetsNames = Lists.newArrayList(fileName);
        List<List<String>> exportDatas = ExcelDownUtil.getTaskExportDatas(param.getExportOptionList(), userTodoList, projectInfoMap);
        // sheet列表集合
        List<List<List<String>>> sheets = Lists.newArrayList();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);
    }

    /**
     * 流程处理相关操作
     *
     * @param param
     * @return void
     * @author CMM
     * @since 2023/02/01
     */
    public void handler(ReqProcessHandlerDTO param) {
        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();
        // 获取入参
        String processInstanceId = param.getInstanceId();
        Long projectId = param.getProjectId();
        VUtils.isTrue(Objects.isNull(processInstanceId)).throwMessage("获取流程实例ID失败!");
        VUtils.isTrue(Objects.isNull(projectId)).throwMessage("获取项目ID失败!");
        // 获取当前申报项目
        Project declaredProject = projectService.getOne(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getInstCode, processInstanceId)
                .eq(Project::getId, projectId));
        VUtils.isTrue(Objects.isNull(declaredProject)).throwMessage("获取项目失败!");
        // 获取当前项目名称
        String projectName = declaredProject.getProjectName();
        // 获取当前项目状态
        Integer projectStatus = declaredProject.getStatus();
        // 获取当前未处理流程详情
        ProcessProgressVo currentInstanceDetail = processInstanceService.getProgressInstanceDetail(null, processInstanceId);

        // 获取当前要处理的流程实例
        HistoricProcessInstance instance = historyService.createHistoricProcessInstanceQuery()
                .processInstanceId(processInstanceId)
                .singleResult();
        // 获取流程发起人信息
        String startUserId = instance.getStartUserId();
        UserInfo startUserInfo = userInfoService.getOne(Wrappers.lambdaQuery(UserInfo.class)
                .eq(UserInfo::getEmployeeCode, startUserId));
        // 获取流程定义名称
        String processDefinitionName = instance.getProcessDefinitionName();

        switch (param.getAction()) {
            // 通过
            case PASS:
            // 盖章并通过
            case SEAL_PASS:
                // 通过该任务，流程到下一审核人处
                processTaskService.handleTask(param, employeeCode);
                // 获取流程通过后的流程实例
                HistoricProcessInstance newInstance = historyService.createHistoricProcessInstanceQuery()
                        .processInstanceId(processInstanceId)
                        .singleResult();
                // 获取流程通过后当前流程详情
                ProcessProgressVo newInstanceDetail = processInstanceService.getProgressInstanceDetail(null, processInstanceId);
                // 获取流程通过后当前审核人信息，向其发送工作通知
                List<ProgressNode> newProgressInfo = newInstanceDetail.getProgressInfo();
                ProgressNode currentNode = newProgressInfo.get(newProgressInfo.size() - 1);
                UserInfo auditUserInfo;
                // 说明当前节点是子流程节点
                if (currentNode.getNodeType().name().equals(NodeTypeEnum.SUB.name())) {
                    List<ProgressNode> children = currentNode.getChildren();
                    // 获取子流程当前审核人节点
                    ProgressNode subCurrentNode = children.get(children.size() - 1);
                    auditUserInfo = userInfoService.getById(Long.valueOf(subCurrentNode.getUserId()));
                } else {
                    auditUserInfo = userInfoService.getById(Long.valueOf(currentNode.getUserId()));
                }

                // 流程通过后，判断当前登录用户是不是最后一个审核人
                // 若当前登录用户是最后一个审批人，需更新流程状态为审核完成，项目状态到下个状态
                // 并向流程发起人发送浙政钉工作通知：【项目名称】已通过【流程名称】，请及时开始下一步操作。
                if (HisProInsEndActId.END.equals(newInstance.getEndActivityId())) {
                    switch (Objects.requireNonNull(ProjectStatusEnum.getValue(projectStatus))) {
                        // 当前项目状态是预审中
                        case PRE_APPLYING:
                            //先修改项目状态
                            updatePassProjectStatus(userId, declaredProject);
                            //然后入库暂存库
                            projectStagingService.addByProject(declaredProject,"暂存入库 待提交部门联审");
                            break;
                        // 当前项目状态是单位内部审核中
                        case UNDER_INTERNAL_AUDIT:
                        // 当前项目状态是部门联审中
                        case DEPARTMENT_JOINT_REVIEW:
                        // 当前项目状态是方案评审中
                        case SCHEME_UNDER_REVIEW:
                        // 当前项目状态是终验审核中
                        case FINAL_ACCEPTANCE_IS_UNDER_REVIEW:
                            updatePassProjectStatus(userId, declaredProject);
                            break;
                        default:
                            throw new IllegalStateException("Unexpected value: " + projectStatus);
                    }
                    // 获取发送浙政钉工作通知必要信息
                    WorkNoticeInfo passWorkNoticeInfo2 = getSendWorkNoticeInfo(startUserInfo);
                    String passMsg2 = String.format(PASS_MSG_TEMPLATE2, projectName, processDefinitionName);
                    passWorkNoticeInfo2.setMsg(passMsg2);
                    // 放入系统通知表中，保存记录
                    assemblyAuditNotify(userId, declaredProject, passMsg2);
                    // 放入工作通知暂存表中，通过扫表异步发送
                    workNoticeStagingService.addByWorkNotice(passWorkNoticeInfo2, MsgTypeEnum.PROJECT_REVIEW);
                } else {
                    // 若有下一个审核人(当前节点的用户)，
                    // 向其发送浙政钉工作通知：标题：审核任务 内容：【单位名称】的【项目名称】需要您审核。
                    // 获取发送浙政钉工作通知必要信息
                    if (Objects.isNull(auditUserInfo)) {
                        throw new BizException("审核人信息不存在!");
                    }
                    WorkNoticeInfo sendWorkNoticeInfo = getSendWorkNoticeInfo(auditUserInfo);
                    String msg = String.format(PASS_MSG_TEMPLATE, sendWorkNoticeInfo.getOrganizationName(), projectName);
                    sendWorkNoticeInfo.setMsg(msg);
                    // 放入系统通知表中，保存记录
                    assemblyAuditNotify(userId, declaredProject, msg);
                    // 放入工作通知暂存表中，通过扫表异步发送
                    workNoticeStagingService.addByWorkNotice(sendWorkNoticeInfo, MsgTypeEnum.PROJECT_REVIEW);
                }
                break;
            // 驳回
            case REJECT:
                // 驳回该任务，中止流程并使项目进入对应状态,给项目创建人、流程发起人发送浙政钉工作通知：
                // 【项目名称】的【流程名称】被驳回，请及时处理。
                processTaskService.handleTask(param, employeeCode);
                // 更新项目状态和流程状态
                updateRejectProjectStatus(userId, declaredProject);
                // 获取发送浙政钉工作通知必要信息
                WorkNoticeInfo rejectWorkNoticeInfo = getSendWorkNoticeInfo(startUserInfo);
                String rejectMsg = String.format(REJECT_MSG_TEMPLATE, projectName, processDefinitionName);
                rejectWorkNoticeInfo.setMsg(rejectMsg);
                // 放入系统通知表中，保存记录
                assemblyAuditNotify(userId, declaredProject, rejectMsg);
                // 放入工作通知暂存表中，通过扫表异步发送
                workNoticeStagingService.addByWorkNotice(rejectWorkNoticeInfo, MsgTypeEnum.PROJECT_REVIEW);
                break;
            // 退回
            case BACK:
                // 退回该任务
                processTaskService.handleTask(param, employeeCode);
                // 给项目创建人、流程发起人发送浙政钉工作通知：【项目名称】的【流程名称】被退回，请及时处理。
                // 获取发送浙政钉工作通知必要信息
                WorkNoticeInfo backWorkNoticeInfo = getSendWorkNoticeInfo(startUserInfo);
                String backMsg = String.format(BACK_MSG_TEMPLATE, projectName, processDefinitionName);
                backWorkNoticeInfo.setMsg(backMsg);
                // 放入系统通知表中，保存记录
                assemblyAuditNotify(userId, declaredProject, backMsg);
                // 放入工作通知暂存表中，通过扫表异步发送
                workNoticeStagingService.addByWorkNotice(backWorkNoticeInfo, MsgTypeEnum.PROJECT_REVIEW);
                break;
            // 撤回(流程发起人和当前流程审核人的前一个审核人操作)
            case WITHDRAW:
                // 登录用户是流程发起人，且是流程发起人撤回
                if (employeeCode.equals(startUserId)
                        && ProcessConstant.Field.ROOT.equals(param.getTaskId())) {
                    processTaskService.handleTask(param, employeeCode);
                    // 若是流程发起人点击撤回，项目回到上一个状态，需调用状态机更新项目状态,流程状态更新为审核通过
                    switch (Objects.requireNonNull(ProjectStatusEnum.getValue(projectStatus))) {
                        // 当前项目状态是单位内部审核中
                        case UNDER_INTERNAL_AUDIT:
                            // 当前项目状态是预审中
                        case PRE_APPLYING:
                            // 当前项目状态是部门联审中
                        case DEPARTMENT_JOINT_REVIEW:
                            // 当前项目状态是方案评审中
                        case SCHEME_UNDER_REVIEW:
                            // 当前项目状态是终验审核中
                        case FINAL_ACCEPTANCE_IS_UNDER_REVIEW:
                            updateWithdrawProjectStatus(userId, declaredProject);
                            break;
                        default:
                            throw new IllegalStateException("Unexpected value: " + projectStatus);
                    }
                } else {
                    // 当前撤回节点不是流程发起人节点
                    List<ProgressNode> currentProgressInfo = currentInstanceDetail.getProgressInfo();
                    ProgressNode progressNode = currentProgressInfo.get(currentProgressInfo.size() - 1);
                    ProgressNode beforeProgressNode;
                    ProgressNode currentProgressNode;
                    // 说明当前节点是子流程节点
                    if (progressNode.getNodeType().name().equals(NodeTypeEnum.SUB.name())) {
                        List<ProgressNode> children = progressNode.getChildren();
                        beforeProgressNode = children.get(children.size() - 2);
                        currentProgressNode = children.get(children.size() - 1);
                    } else {
                        beforeProgressNode = currentProgressInfo.get(currentProgressInfo.size() - 2);
                        currentProgressNode = currentProgressInfo.get(currentProgressInfo.size() - 1);
                    }
                    // 获取当前工作流任务前一个审核人信息
                    ProcessInstanceUserDto beforeUser = beforeProgressNode.getUser();
                    UserFullInfoDTO beforeUserInfo = userInfoHelper.getUserFullInfo(Long.valueOf(beforeUser.getUserId()));
                    // 获取当前当前工作流任务当前审核人信息
                    ProcessInstanceUserDto currentUser = currentProgressNode.getUser();
                    UserFullInfoDTO currentUserInfo = userInfoHelper.getUserFullInfo(Long.valueOf(currentUser.getUserId()));
                    // 判断当前工作流任务前一个审核人的部门和当前登录用户的部门是否是同一个，如果是同一个才可以撤回，否则抛出异常
                    boolean orgFlag = currentUserInfo.getOrganizationCode().equals(beforeUserInfo.getOrganizationCode());
                    boolean userFlag = beforeUser.getUserId().equals(String.valueOf(userId));
                    if (!orgFlag) {
                        throw new BizException("下一个审核人和您不是同一个部门,无法撤回!");
                    } else if (!userFlag) {
                        throw new BizException("当前登录用户无法进行撤回操作!");
                    } else {
                        processTaskService.handleTask(param, employeeCode);
                    }
                }
                break;
            default:
                throw new IllegalStateException("Unexpected value: " + param.getAction());
        }
    }

    /**
     * 装配项目审核工作通知
     * @param userId
     * @param project
     * @param msg
     */
    private void assemblyAuditNotify(Long userId, Project project, String msg) {
        Notify notify = new Notify();
        notify.setTitle(msg);
        notify.setUserId(userId);
        notify.setType(MsgTypeEnum.PROJECT_REVIEW.name());
        notify.setInstanceId(project.getInstCode());
        notify.setContent(msg);
        notify.setReaded(Boolean.FALSE);
        notify.setCreateTime(LocalDateTime.now());
        notifyService.save(notify);
    }

    /**
     * 调用IRS接口,获取盖章后的pdf文件,上传到OSS，并保存文件ID到项目库中
     * @param req
     * @param declaredProject
     * @return void
     * @author CMM
     * @since 2023/03/16 21:34
     */
    private Long getSealedPdf(SealInfoDTO req, Project declaredProject) {
        Long projectId = req.getProjectId();
        // 设置pdf模板参数
        JSONObject paramsMap = assemblyPdfParams(declaredProject);
        // TODO 根据当前盖章单位设置对应的盖章意见与盖章日期
        paramsMap.put("superOrgOpinion",null);
        paramsMap.put("superOrgAuditDate",null);
        paramsMap.put("higherOrgOpinion",null);
        paramsMap.put("higherOrgAuditDate",null);
        paramsMap.put("bigDataBureauOpinion",null);
        paramsMap.put("bigDataBureauAuditDate",null);
        // 再次获取装配了盖章意见和盖章日期参数的未盖章pdf文件
        Long pretrialFileId = generateNotSealedPdf(projectId,paramsMap);
        // 获取预审申请单文件,并转换为Base64
        File pdfFile = fileService.getById(pretrialFileId);
        InputStream fileInputStream = fileService.getFileInputStream(pdfFile);
        String pdfEncode64 = Base64.encode(fileInputStream);
        // 装配盖章接口请求参数信息
        SignReqDTO signReq = new SignReqDTO();
        signReq.setPdfEncode64(pdfEncode64);
        String originalFileName = pdfFile.getOriginalFileName();
        signReq.setFileName(originalFileName);
        Float posX = req.getPosX();
        Float posY = req.getPosY();
        signReq.setPosX(posX);
        signReq.setPosY(posY);
        // 获取当前登录签章用户信息
        Long userId = LoginUserUtil.getUserId();
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取印章编号
        CompanySignature companySignature = companySignatureService.getOne(Wrappers.lambdaQuery(CompanySignature.class)
                .eq(CompanySignature::getOrganizationCode, userFullInfo.getOrganizationCode()));
        String sealSn = companySignature.getSealSn();
        signReq.setSealSn(sealSn);
        String posPage = req.getPosPage();
        signReq.setPosPage(posPage);
        Integer signType = req.getSignType();
        signReq.setSignType(signType);
        // 调用盖章接口,获取盖章后返回的pdf文件字符数组
        byte[] signPdf = IRSAPIRequest.createSignPdf(signReq);
        // 转换成MultipartFile
        MultipartFile multipartFile = new MockMultipartFile("file", originalFileName + ".pdf", "application/pdf", signPdf);
        // 上传OSS
        FileResultVO resultVo = fileService.upload(multipartFile, "default");
        // 将返回的文件ID更新到项目库中对应的项目下
        declaredProject.setPretrialFileId(resultVo.getId());
        declaredProject.setPretrialFileName(resultVo.getOriginalFileName());
        declaredProject.setUpdateOn(LocalDateTime.now());
        projectService.updateById(declaredProject);
        return resultVo.getId();
    }


    /**
     * 当为驳回操作时，更新项目表中的项目状态
     *
     * @param userId
     * @param declaredProject
     * @return void
     * @author CMM
     * @since 2023/02/08
     */
    private void updateRejectProjectStatus(Long userId, Project declaredProject) {
        stateMachineUtils.reject(declaredProject);
        declaredProject.setUpdateOn(LocalDateTime.now());
        declaredProject.setUpdateBy(userId);
        projectService.updateById(declaredProject);
    }


    /**
     * 获取发送浙政钉工作通知的信息
     *
     * @param auditUserInfo
     * @return com.ningdatech.pmapi.todocenter.bean.entity.WorkNoticeInfo
     * @author CMM
     * @since 2023/02/15 14:04
     */
    public WorkNoticeInfo getSendWorkNoticeInfo(UserInfo auditUserInfo) {
        WorkNoticeInfo workNoticeInfo = new WorkNoticeInfo();
        Long accountId = auditUserInfo.getAccountId();
        if (Objects.isNull(accountId)) {
            throw new BizException("该用户没有录入浙政钉信息!");
        }
        workNoticeInfo.setAccountId(accountId);
        // 根据浙政钉用户ID获取部门code
        DingEmployeeInfo employeeInfo = dingEmployeeInfoService.getOne(Wrappers.lambdaQuery(DingEmployeeInfo.class)
                .eq(DingEmployeeInfo::getAccountId, accountId)
                .eq(DingEmployeeInfo::getMainJob,String.valueOf(Boolean.TRUE))
                .last("limit 1"));
        String organizationCode = employeeInfo.getOrganizationCode();
        workNoticeInfo.setOrganizationCode(organizationCode);
        // 根据部门code获取部门名称
        DingOrganization dingOrganization = dingOrganizationService.getOne(Wrappers.lambdaQuery(DingOrganization.class)
                .eq(DingOrganization::getOrganizationCode, organizationCode));
        String organizationName = dingOrganization.getOrganizationName();
        workNoticeInfo.setOrganizationName(organizationName);
        // 构建唯一的消息ID
        String bizMsgId = "ZD_WORK_NOTICE_" + StrUtil.UNDERLINE + organizationCode + StrUtil.UNDERLINE
                + organizationName + accountId + StrUtil.UNDERLINE + System.currentTimeMillis();
        workNoticeInfo.setBizMsgId(bizMsgId);
        String receiverUserId = String.valueOf(accountId);
        workNoticeInfo.setReceiverUserId(receiverUserId);
        return workNoticeInfo;
    }

    /**
     * 当为通过操作时，更新项目表中项目状态
     *
     * @param userId
     * @param declaredProject
     * @return void
     * @author CMM
     * @since 2023/02/08
     */
    private void updatePassProjectStatus(Long userId, Project declaredProject) {
        stateMachineUtils.pass(declaredProject);
        declaredProject.setUpdateOn(LocalDateTime.now());
        declaredProject.setUpdateBy(userId);
        projectService.updateById(declaredProject);
    }

    /**
     * 获取流程发起节点的用户ID
     *
     * @param rootNode          根节点
     * @param processInstanceId
     * @return java.lang.String
     * @author CMM
     * @since 2023/02/02
     */
    private String getRootUserId(FlowNode rootNode, String processInstanceId) {
        String rootUserId = null;
        // 输出连线
        List<SequenceFlow> outgoingFlows = rootNode.getOutgoingFlows();
        // 遍历返回下一个节点信息
        for (SequenceFlow currentOutgoingFlow : outgoingFlows) {
            // 类型自己判断
            FlowElement targetFlowElement = currentOutgoingFlow.getTargetFlowElement();
            // TODO 若要会签需判断候选人
            // 发起事件
            if (targetFlowElement instanceof StartEvent) {
                String actId = targetFlowElement.getId();
                ActivityInstance activityInstance = runtimeService.createActivityInstanceQuery()
                        .processInstanceId(processInstanceId).activityId(actId).singleResult();
                String executionId = activityInstance.getExecutionId();
                rootUserId = runtimeService.getVariable(executionId, "initiator", String.class);
                break;
            }
        }
        return rootUserId;

    }

    /**
     * 获取当前节点的下一个节点的审核用户ID
     *
     * @param currentNode       当前节点
     * @param processInstanceId
     * @return java.lang.String 下一个节点的审核用户ID
     * @author CMM
     * @since 2023/02/02
     */
    private String getNextUserId(FlowNode currentNode, String processInstanceId) {
        String nextUserId = null;
        // 输出连线
        List<SequenceFlow> outgoingFlows = currentNode.getOutgoingFlows();
        // 遍历返回下一个节点信息
        for (SequenceFlow currentOutgoingFlow : outgoingFlows) {
            // 类型自己判断
            FlowElement targetFlowElement = currentOutgoingFlow.getTargetFlowElement();
            // 如果下个审批节点为结束节点,那么跳过该节点
            if (targetFlowElement instanceof EndEvent) {
                continue;
            }
            // TODO 若要会签需判断候选人
            // 用户任务
            if (targetFlowElement instanceof UserTask) {
                String actId = targetFlowElement.getId();
                ActivityInstance activityInstance = runtimeService.createActivityInstanceQuery()
                        .processInstanceId(processInstanceId).activityId(actId).singleResult();
                String executionId = activityInstance.getExecutionId();
                nextUserId = runtimeService.getVariable(executionId, "assignee", String.class);
                break;
            }
        }
        return nextUserId;
    }

    /**
     * 当为撤回操作时，更新项目表中的项目状态为前一个状态
     *
     * @param userId
     * @param declaredProject
     * @return void
     * @author CMM
     * @since 2023/02/08
     */
    private void updateWithdrawProjectStatus(Long userId, Project declaredProject) {
        try {
            stateMachineUtils.withDraw(declaredProject);
            declaredProject.setUpdateOn(LocalDateTime.now());
            declaredProject.setUpdateBy(userId);
        } catch (Exception e) {
            throw new BizException("状态机执行失败!");
        }
    }

    /**
     * 查询流程表单数据及审批的进度步骤
     *
     * @param request 请求参数
     * @return 流程进度及表单详情
     */
    public ProcessProgressDetailVo getProcessDetail(ProcessDetailReq request) {
        String instanceId = request.getInstanceId();
        String nodeId = request.getNodeId();
        Long projectId = request.getProjectId();
        ProcessProgressVo progressInstanceDetail = processInstanceService.getProgressInstanceDetail(nodeId, instanceId);

        List<ProgressNode> progressInfo = progressInstanceDetail.getProgressInfo();
        buildUserUtils.buildUserByProcessInfo(progressInfo);

        ProcessProgressDetailVo res = new ProcessProgressDetailVo();
        res.setProcessProgressVo(progressInstanceDetail);
        res.setStatus(progressInstanceDetail.getStatus());
        res.setProjectId(projectId);
        return res;
    }


    /**
     * 待办中心我已处理项目列表查询
     *
     * @param param
     * @author CMM
     * @since 2023/02/03
     */
    public PageVo<ResToBeProcessedVO> idoList(ToBeProcessedReq param) {

        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
        // 再查出项目关联的流程实例ID
        List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
        if (CollUtil.isEmpty(projectIdList)) {
            return PageVo.empty();
        }
        List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIdList)
                .orderByDesc(ProjectInst::getProjectId));
        Map<String, Project> projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
        List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

        // 查出用户工作流
        TodoCenterListReqDTO req = new TodoCenterListReqDTO();
        Integer pageNumber = param.getPageNumber();
        Integer pageSize = param.getPageSize();
        req.setProcessDefId(param.getProcessDefId());
        req.setUserId(String.valueOf(userId));
        req.setEmployeeCode(employeeCode);
        req.setInstCodes(instCodes);
        List<ProcessTaskVo> iDoList = processTaskService.getUserIdoList(req);
        if (CollUtil.isEmpty(iDoList)) {
            return PageVo.empty();
        }

        List<ProcessTaskVo> userIdoList = iDoList.stream()
                .skip((long) (pageNumber - 1) * pageSize)
                .limit(pageSize)
                .collect(Collectors.toList());

        List<ResToBeProcessedVO> resVos = userIdoList.stream().map(d -> {
            ResToBeProcessedVO res = new ResToBeProcessedVO();
            Project projectInfo = projectInfoMap.get(d.getInstanceId());
            BeanUtils.copyProperties(projectInfo, res);
            res.setBuildOrg(projectInfo.getBuildOrgName());
            res.setDeclaredAmount(projectInfo.getDeclareAmount());
            res.setProjectId(projectInfo.getId());
            res.setProcessStatusName(d.getStatus());
            res.setProcessLaunchTime(d.getCreateTime());
            res.setProcessHandleTime(d.getTaskEndTime());
            return res;
        }).collect(Collectors.toList());
        return PageVo.of(resVos, iDoList.size());
    }

    /**
     * 导出我已处理项目列表
     *
     * @param response 返回数据流
     * @param param    请求参数
     * @return void
     * @author CMM
     * @since 2023/02/06
     */
    public void idoExport(HttpServletResponse response, ToBeProcessedExportReq param) {
        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        //因为要解耦 不能把项目和工作流一起查
        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        List<ProcessTaskVo> userIdoList = Lists.newArrayList();
        Map<String, Project> projectInfoMap = Maps.newHashMap();
        if (CollUtil.isEmpty(projects)) {
            Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
            // 再查出项目关联的流程实例ID
            List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());

            List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                    .in(ProjectInst::getProjectId, projectIdList)
                    .orderByDesc(ProjectInst::getProjectId));
            projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
            List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

            // 查出用户工作流
            TodoCenterListReqDTO req = new TodoCenterListReqDTO();
            req.setProcessDefId(param.getProcessDefId());
            req.setUserId(String.valueOf(userId));
            req.setEmployeeCode(employeeCode);
            req.setInstCodes(instCodes);
            userIdoList = processTaskService.getUserIdoList(req);
        }

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();

        String fileName = null;
        if (IsAppendProjectEnum.APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_我已处理_增补项目列表";
        } else if (IsAppendProjectEnum.NOT_APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_我已处理_非增补项目列表";
        }
        excelExportWriter.setFileName(fileName);

        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);

        List<List<String>> exportDatas = ExcelDownUtil.getTaskExportDatas(param.getExportOptionList(), userIdoList, projectInfoMap);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);

    }

    /**
     * 查询我提交的项目列表
     *
     * @param param
     * @return com.ningdatech.basic.model.PageVo<com.ningdatech.pmapi.todocenter.model.dto.res.ResToBeProcessedDTO>
     * @author CMM
     * @since 2023/02/06
     */
    public PageVo<ResToBeProcessedVO> mySubmittedList(ToBeProcessedReq param) {
        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);
        Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
        // 再查出项目关联的流程实例ID
        List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());

        if (CollUtil.isEmpty(projectIdList)) {
            return PageVo.empty();
        }

        List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIdList)
                .orderByDesc(ProjectInst::getProjectId));
        Map<String, Project> projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
        List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

        // 查出用户工作流
        TodoCenterListReqDTO req = new TodoCenterListReqDTO();
        Integer pageNumber = param.getPageNumber();
        Integer pageSize = param.getPageSize();
        req.setProcessDefId(param.getProcessDefId());
        req.setUserId(String.valueOf(userId));
        req.setEmployeeCode(employeeCode);
        req.setInstCodes(instCodes);
        List<ProcessInstanceVo> submittedList = processInstanceService.getUserSubmittedList(req);

        if (CollUtil.isEmpty(submittedList)) {
            return PageVo.empty();
        }

        List<ProcessInstanceVo> userSubmittedList = submittedList.stream()
                .skip((long) (pageNumber - 1) * pageSize)
                .limit(pageSize)
                .collect(Collectors.toList());

        List<ResToBeProcessedVO> resVos = userSubmittedList.stream().map(d -> {
            ResToBeProcessedVO res = new ResToBeProcessedVO();
            Project project = projectInfoMap.get(d.getInstanceId());
            BeanUtils.copyProperties(project, res);
            res.setBuildOrg(project.getBuildOrgName());
            res.setDeclaredAmount(project.getDeclareAmount());
            res.setProjectId(project.getId());
            res.setProcessStatusName(d.getStatus());
            res.setProcessLaunchTime(d.getStartTime());
            return res;
        }).collect(Collectors.toList());
        return PageVo.of(resVos,submittedList.size());
    }

    /**
     * 待办中心-我发起的项目列表导出
     *
     * @param response
     * @param param
     * @return void
     * @author CMM
     * @since 2023/02/11
     */
    public void exportMySubmittedList(HttpServletResponse response, ToBeProcessedExportReq param) {

        // 获取登录用户ID
        long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);

        List<ProcessInstanceVo> userSubmittedList = Lists.newArrayList();
        Map<String, Project> projectInfoMap = Maps.newHashMap();
        if (CollUtil.isEmpty(projects)) {
            Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
            // 再查出项目关联的流程实例ID
            List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
            List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                    .in(ProjectInst::getProjectId, projectIdList)
                    .orderByDesc(ProjectInst::getProjectId));
            projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
            List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

            // 查出用户工作流
            TodoCenterListReqDTO req = new TodoCenterListReqDTO();
            req.setProcessDefId(param.getProcessDefId());
            req.setUserId(String.valueOf(userId));
            req.setEmployeeCode(employeeCode);
            req.setInstCodes(instCodes);
            userSubmittedList = processInstanceService.getUserSubmittedList(req);
        }

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();

        String fileName = null;
        if (IsAppendProjectEnum.APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_我发起的_增补项目列表";
        } else if (IsAppendProjectEnum.NOT_APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_我发起的_非增补项目列表";
        }
        excelExportWriter.setFileName(fileName);

        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);
        // 表体行数据集合
        List<List<String>> exportDatas = ExcelDownUtil.getInstExportDatas(param.getExportOptionList(), userSubmittedList, projectInfoMap);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);

    }

    /**
     * 待办中心-抄送我的项目列表查询
     *
     * @param param
     * @return com.ningdatech.basic.model.PageVo<com.ningdatech.pmapi.todocenter.model.dto.res.ResToBeProcessedDTO>
     * @author CMM
     * @since 2023/02/11
     */
    public PageVo<ResToBeProcessedVO> ccmeList(ToBeProcessedReq param) {

        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);

        if (CollUtil.isEmpty(projects)) {
            return PageVo.empty();
        }

        Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
        // 再查出项目关联的流程实例ID
        List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
        List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                .in(ProjectInst::getProjectId, projectIdList)
                .orderByDesc(ProjectInst::getProjectId));
        Map<String, Project> projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
        List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

        // 查出用户工作流
        TodoCenterListReqDTO req = new TodoCenterListReqDTO();
        Integer pageNumber = param.getPageNumber();
        Integer pageSize = param.getPageSize();
        req.setProcessDefId(param.getProcessDefId());
        req.setUserId(String.valueOf(userId));
        req.setEmployeeCode(employeeCode);
        req.setInstCodes(instCodes);
        BeanUtils.copyProperties(param,req);
        List<ProcessInstanceVo> ccMeList = processInstanceService.getCcMeList(req);

        if (CollUtil.isEmpty(ccMeList)) {
            return PageVo.empty();
        }

        List<ProcessInstanceVo> ccMeInstList = ccMeList.stream()
                .skip((long) (pageNumber - 1) * pageSize)
                .limit(pageSize)
                .collect(Collectors.toList());

        List<ResToBeProcessedVO> resVos = ccMeInstList.stream().map(d -> {
            ResToBeProcessedVO res = new ResToBeProcessedVO();
            Project project = projectInfoMap.get(d.getInstanceId());
            BeanUtils.copyProperties(project, res);
            res.setNodeId(d.getNodeId());
            res.setProjectId(project.getId());
            res.setBuildOrg(project.getBuildOrgName());
            res.setDeclaredAmount(project.getDeclareAmount());
            res.setProcessStatusName(d.getStatus());
            res.setProcessLaunchTime(d.getStartTime());
            return res;
        }).collect(Collectors.toList());
        return PageVo.of(resVos, ccMeList.size());
    }

    /**
     * 待办中心-抄送我的项目列表导出
     *
     * @param response
     * @param param
     * @return void
     * @author CMM
     * @since 2023/02/11
     */
    public void exportCcMeProjectList(HttpServletResponse response, ToBeProcessedExportReq param) {

        // 获取登录用户ID
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        // 查出项目库项目
        ProjectListReq projectListReq = new ProjectListReq();
        BeanUtils.copyProperties(param, projectListReq);
        List<Project> projects = projectLibManage.projectList(projectListReq);

        List<ProcessInstanceVo> ccMeList = Lists.newArrayList();
        Map<String, Project> projectInfoMap = Maps.newHashMap();
        if (CollUtil.isEmpty(projects)) {
            Map<Long, Project> projectsMap = projects.stream().collect(Collectors.toMap(Project::getId, v -> v));
            // 再查出项目关联的流程实例ID
            List<Long> projectIdList = projects.stream().map(Project::getId).collect(Collectors.toList());
            List<ProjectInst> projectInstList = projectInstService.list(Wrappers.lambdaQuery(ProjectInst.class)
                    .in(ProjectInst::getProjectId, projectIdList)
                    .orderByDesc(ProjectInst::getProjectId));
            projectInfoMap = projectInstList.stream().collect(Collectors.toMap(ProjectInst::getInstCode, p-> projectsMap.get(p.getProjectId())));
            List<String> instCodes = projectInstList.stream().map(ProjectInst::getInstCode).collect(Collectors.toList());

            // 分页查出用户工作流
            TodoCenterListReqDTO req = new TodoCenterListReqDTO();
            req.setProcessDefId(param.getProcessDefId());
            req.setUserId(String.valueOf(userId));
            req.setEmployeeCode(employeeCode);
            req.setInstCodes(instCodes);
            BeanUtils.copyProperties(param,req);
            ccMeList = processInstanceService.getCcMeList(req);
        }

        ExcelExportWriter excelExportWriter = new ExcelExportWriter();

        String fileName = null;
        if (IsAppendProjectEnum.APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_抄送我的_增补项目列表";
        } else if (IsAppendProjectEnum.NOT_APPEND_PROJECT.getCode().equals(param.getIsTemporaryAugment())) {
            fileName = "待办中心_抄送我的_非增补项目列表";
        }
        excelExportWriter.setFileName(fileName);

        List<String> sheetsNames = new ArrayList<>();
        sheetsNames.add(fileName);
        // 表体行数据集合
        List<List<String>> exportDatas = ExcelDownUtil.getInstExportDatas(param.getExportOptionList(), ccMeList, projectInfoMap);
        // sheet列表集合
        List<List<List<String>>> sheets = new ArrayList<>();
        sheets.add(exportDatas);
        excelExportWriter.setHeads(sheets);
        excelExportWriter.setDatas(exportDatas);
        excelExportWriter.setSheets(sheetsNames);
        excelExportWriter.setHeadPropertyClass(List.class);
        excelExportWriter.setHeadPropertyFlag(Boolean.FALSE);
        ExcelDownUtil.excelExportFile(response, excelExportWriter);

    }

    /**
     * 被退回项目内容调整并重新通过
     * @param dto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String adjustmentAndHandle(AdjustHandleDTO dto) {
        ProjectDTO projectDto = dto.getProjectInfo();
        Project projectInfo = projectService.getById(projectDto.getId());

        VUtils.isTrue(Objects.isNull(projectInfo)).throwMessage("调整失败 此项目不存在!");

        //项目名称去重
        defaultDeclaredProjectManage.checkDuplication(projectDto);

        //修改项目内容
        if(!modifyProject(projectDto)){
            throw new BusinessException("调整项目失败!");
        }

        //最后去审批(通过)当前退回节点的审核任务
        ReqProcessHandlerDTO handlerDTO = new ReqProcessHandlerDTO();
        handlerDTO.setProjectId(dto.getProjectId());
        handlerDTO.setInstanceId(dto.getInstanceId());
        handlerDTO.setTaskId(dto.getTaskId());
        handlerDTO.setAction(ProcessHandlerEnum.PASS);
        handler(handlerDTO);
        return dto.getInstanceId();

    }

    private Boolean modifyProject(ProjectDTO projectDto) {
        //先修改项目信息
        Project project = new Project();
        BeanUtils.copyProperties(projectDto,project);
        project.setUpdateOn(LocalDateTime.now());
        if(!projectService.updateById(project)){
            throw new BusinessException("项目调整失败");
        }
        //再修改应用信息
        if(CollUtil.isNotEmpty(projectDto.getApplicationList())){
            //采取批量删除 批量添加的方式
            projectApplicationService.remove(Wrappers.lambdaQuery(ProjectApplication.class)
                    .eq(ProjectApplication::getProjectId,project.getId()));
            //批量添加
            List<ProjectApplication> applications = projectDto.getApplicationList().stream().map(application -> {
                ProjectApplication projectApplication = new ProjectApplication();
                BeanUtils.copyProperties(application, projectApplication);
                projectApplication.setProjectId(project.getId());
                return projectApplication;
            }).collect(Collectors.toList());
            projectApplicationService.saveBatch(applications);
        }
        return Boolean.TRUE;
    }

    /**
     * 获取未盖章的预审申请单pdf文件
     * @param projectId
     * @return java.lang.Long
     * @author CMM
     * @since 2023/03/16 22:46
     */
    public Long getNotSealedPdf(Long projectId) {
        // 设置pdf模板参数
        Project project = projectService.getById(projectId);
        JSONObject paramsMap = assemblyPdfParams(project);
        paramsMap.put("superOrgOpinion",null);
        paramsMap.put("superOrgAuditDate",null);
        paramsMap.put("higherOrgOpinion",null);
        paramsMap.put("higherOrgAuditDate",null);
        paramsMap.put("bigDataBureauOpinion",null);
        paramsMap.put("bigDataBureauAuditDate",null);
        return generateNotSealedPdf(projectId, paramsMap);
    }

    private Long generateNotSealedPdf(Long projectId, JSONObject paramsMap) {
        // 获取本地目录的pdf模板
        String fileName = "预审申请单";
        InputStream htmlInputStream =
                this.getClass().getClassLoader().getResourceAsStream("template" + java.io.File.separator + fileName + ".html");
        // 生成pdf字节数组
        byte[] pdf = pdfUtils.generatePdf(htmlInputStream, paramsMap);

        // 转换成MultipartFile
        MultipartFile multipartFile = new MockMultipartFile("file", fileName + ".pdf", "application/pdf", pdf);

        // 上传OSS
        FileResultVO resultVO = fileService.upload(multipartFile, "default");

        // 将返回的文件ID保存到项目库中
        Project project = projectService.getById(projectId);
        project.setPretrialFileId(resultVO.getId());
        project.setPretrialFileName(resultVO.getOriginalFileName());
        project.setUpdateOn(LocalDateTime.now());
        projectService.updateById(project);
        try {
            if (null != htmlInputStream) {
                htmlInputStream.close();
            }

            // 删除输出文件夹下的临时文件
            java.io.File directory = new java.io.File("");
            //pdf输出路径
            String filePath = directory.getAbsolutePath() + java.io.File.separator + "fileout";
            java.io.File directoryPath = new java.io.File(filePath);
            FileUtils.cleanDirectory(directoryPath);
        } catch (IOException e) {
            log.error(e.getMessage());
        }

        // 返回文件id供前端下载
        return resultVO.getId();
    }

    /**
     * 装配pdf模板参数
     * @param project 申报项目ID
     * @return com.alibaba.fastjson.JSONObject
     * @author CMM
     * @since 2023/03/16 22:28
     */
    private JSONObject assemblyPdfParams(Project project) {
        // 获取本地目录的pdf模板
        String fileName = "预审申请单";
        Long projectId = project.getId();
        // 获取表单数据
        PdfGenerateDTO pdfGenerateDTO = new PdfGenerateDTO();
        BeanUtils.copyProperties(project, pdfGenerateDTO);

        JSONObject paramsMap = JSONObject.parseObject(JSONObject.toJSONString(pdfGenerateDTO));
        // 设置title
        String title = "【" + project.getProjectName() + "】" + fileName;
        paramsMap.put("title",title);
        paramsMap.put("projectNo", projectId);
        // 获取预审申报的开始时间
        ProjectInst projectInst = projectInstService.getOne(Wrappers.lambdaQuery(ProjectInst.class)
                .eq(ProjectInst::getProjectId, projectId)
                .eq(ProjectInst::getInstType, InstTypeEnum.PRELIMINARY_PREVIEW.getCode())
                .orderByDesc(ProjectInst::getCreatOn)
                .last("limit 1"));
        LocalDateTime time = null;
        if (Objects.nonNull(projectInst)) {
            time = projectInst.getCreatOn();
            paramsMap.put("time", NdDateUtils.format(time, "yyyy-MM-dd HH:mm"));
        }else {
            paramsMap.put("time", null);
        }
        // 获取是否临时增补项目
        String isTemporaryAugment = IsOrNotEnum.getDescByCode(project.getIsTemporaryAugment());
        paramsMap.put("isTemporaryAugment", isTemporaryAugment);
        // 获取项目类型
        Integer projectType = project.getProjectType();
        paramsMap.put("projectType", ProjectTypeEnum.getDesc(projectType));
        // 获取是否首次新建
        String isFirst = IsOrNotEnum.getDescByCode(project.getIsFirst());
        paramsMap.put("isFirst", isFirst);
        // 获取是否一地创新全省共享项目
        Boolean isInnovateWholeProvinceShare = project.getIsInnovateWholeProvinceShare();
        paramsMap.put("isInnovateWholeProvinceShare", Boolean.TRUE.equals(isInnovateWholeProvinceShare) ?
                IsOrNotEnum.IS.getDesc() : IsOrNotEnum.NOT.getDesc());
        // 获取项目建设起始时间
        String beginTime = project.getBeginTime();
        String endTime = project.getEndTime();
        String beginAndEndTime = beginTime + StrUtil.DASHED + endTime;
        paramsMap.put("beginAndEndTime", beginAndEndTime);
        // 获取四大体系
        String fourSystem = FourSystemEnum.getDescByCode(project.getFourSystems());
        paramsMap.put("fourSystems", fourSystem);
        // 获取是否数字化改革项目
        String isDigitalReform = IsOrNotEnum.getDescByCode(project.getIsDigitalReform());
        paramsMap.put("isDigitalReform", isDigitalReform);
        // 获取是否上云
        String isCloud = IsOrNotEnum.getDescByCode(project.getIsCloud());
        paramsMap.put("isCloud", isCloud);
        // 获取本年计划投资金额
        BigDecimal yearPlanInvest = project.getAnnualPlanAmount();
        paramsMap.put("yearPlanInvest",yearPlanInvest);
        // TODO 根据地区编码获取当地大数据局(中心)的名称
        paramsMap.put("bigDataBureauName",null);

        // 获取立项依据
        String buildBasis = project.getBuildBasis();
        StringBuilder builder = new StringBuilder();
        List<JSONObject> fileArray = JSON.parseArray(buildBasis, JSONObject.class);
        for (JSONObject jsonObject : fileArray) {
            String titleStr = jsonObject.getString(CommonConst.TITLE);
            builder.append(CommonConst.ITEM_BASED);
            builder.append(titleStr);
            builder.append(StrPool.SPACE);
            String fileNameStr = jsonObject.getString(CommonConst.FILE_NAME_STR);
            builder.append(CommonConst.ITEM_BASED_FILE_NAME);
            builder.append(fileNameStr);
            builder.append(StrPool.SPACE);
            String fileInfo = jsonObject.getString(CommonConst.FILE_LIST);
            List<JSONObject> fileInfoArray = JSON.parseArray(fileInfo, JSONObject.class);
            List<String> nameList = CollUtils.fieldList(fileInfoArray, w -> w.getString(CommonConst.FILE_NAME));
            String appendixStr = nameList.stream().collect(Collectors.joining(StrPool.COMMA));
            builder.append(CommonConst.APPENDIX);
            builder.append(appendixStr);
            builder.append(StrPool.SEMICOLON);
            builder.append(StrPool.NEWLINE);
        }
        String buildBasisList = builder.toString();
        paramsMap.put("buildBasisList",buildBasisList);
        return paramsMap;
    }

    public Long getSealedPdf(SealInfoDTO req) {
        Long projectId = req.getProjectId();
        Project declaredProject = projectService.getById(projectId);
        return getSealedPdf(req, declaredProject);
    }

    //获取当前登录人的 待办中心统计数据
    public TodoCenterStatisticsVO todoCenterStatistics(){
        Long userId = LoginUserUtil.getUserId();
        // 获取登录用户全量信息
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        // 获取登录用户浙政钉code
        String employeeCode = userFullInfo.getEmployeeCode();

        //为了查询效率 还是自己去写统计接口
        TodoCenterStatisticsPO statistics = statisticsService.getStatistics(employeeCode);
        TodoCenterStatisticsVO statisticsVO = BeanUtil.copyProperties(statistics,TodoCenterStatisticsVO.class);

        return statisticsVO;
    }
}
